<%- include('../header') -%>

<h1><%= title %></h1>

<div id="users">
  <% users.forEach(function(user, index) { %>
    <li>
      <a href="/user/<%= index %>"><%= user.name %></a>
      <a href="/user/<%= index %>/edit">edit</a>
    </li>
  <% }) %>
</div>

<%- include('../footer') -%>
