<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    <title>Internal Server Error</title>
    <link rel="stylesheet" href="/style.css">
  </head>
  <body>
    <h1>500: Internal Server Error</h1>
    <p>Looks like something blew up!</p>
  </body>
</html>
