<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width,initial-scale=1">
  <link rel="stylesheet" href="/style.css">
  <title>Edit {{user.name}}</title>
</head>
<body>
  <h1>{{user.name}}</h1>
  <form action="/user/{{user.id}}?_method=put" method="post">
    <label for="user[name]">Name:
      <input type="text" name="user[name]" value="{{user.name}}">
    </label>

    <input type="submit" name="submit" value="Update">
  </form>

  <form action="/user/{{user.id}}/pet" method="post">
    <label for="pet[name]">Pet:
      <input type="text" name="pet[name]" placeholder="Pet Name">
    </label>

    <input type="submit" name="submit" value="Add">
  </form>
</body>
</html>
