<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    <title>Not Found</title>
    <link rel="stylesheet" href="/style.css">
  </head>
  <body>
    <h1>404: Not Found</h1>
    <p>Sorry we can't find <%= url %></p>
  </body>
</html>
