<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width,initial-scale=1">
  <link rel="stylesheet" href="/style.css">
  <title>Users</title>
</head>
<body>
  <h1>Users</h1>
  <p>Click a user below to view their pets.</p>
  <ul>
    {{#each users}}
      <li><a href="/user/{{id}}">{{name}}</a></li>
    {{/each}}
  </ul>
</body>
</html>
