<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    <title><%= title %></title>
    <style media="screen">
      body {
        padding: 50px;
        font: 16px Helvetica, Arial;
      }
    </style>
  </head>
  <body>
    <h2><%= title %></h2>
    <% users.forEach(function(user) { %>
      <li><strong><%= user.name %></strong> is a <% user.age %> year old <%= user.species %></li>
    <% }); %>
  </body>
</html>
